/*
  A simple program for controlling lpt on a low level
  Copyright (C) 2008-2009  Patrik Lembke
                                    
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <sys/io.h>
#include <math.h>

#define VERSION "0.0.2"
#define BASE 0x378

void set_pin( uint pin, bool state );
void print_pins( void );

int main( int argc, char **argv )
{
    uint pin;
    bool state;

    printf( "ppio v%s\n", VERSION );
    printf( "Copyright (C) 2008-2009  Patrik Lembke.\n" );
    printf( "This is free software with ABSOLUTELY NO WARRANTY.\n\n" );

    /* Get ioperms! */
    if( ioperm( BASE, 3, 1 ) )
    {
        fprintf( stderr, "Error: can't access 0x%x, sure you are root?\n",
                 BASE );
        exit( EXIT_FAILURE );
    }

    if( argc == 2 )
    {
        if( strncmp( argv[1], "reset", 5 ) == 0 )
        {
            /* Set all outputs LOW */
            printf( "Reseting!\n" );
            outb( 0, BASE ); /* Low? */
        }
        
        else if( strncmp( argv[1], "status", 6 ) == 0 )
        {
            print_pins();
        }
    }

    else if( argc != 3 )
    {
        printf( "Usage: ppio [pin] [1/0]\n" );
        printf( "\nExample: ppio 1 1\n" );
        printf( "This sets output pin high\n" );
        printf( "\nExample: ppio reset\nSets all outputs low\n" );
        printf( "To just get a look at current output status:\n" );
        printf( "\nExample: ppio status\n" );
        exit( EXIT_FAILURE );
    }

    else
    {
        /* Convert! */
        pin = atoi( argv[1] );
        state = atoi( argv[2] );

        if( 1 > pin || pin > 8 )
        {
            fprintf( stderr, "Error: pin must be between 1 and 8\n" );
            exit( EXIT_FAILURE );
        }

        printf( "Setting pin %d ", pin );

        if( state )
            printf( "HIGH\n" );
        
        else
            printf( "LOW\n" );

        set_pin( pin, state );
    }

    exit( EXIT_SUCCESS );
}

void set_pin( uint pin, bool state )
{
    int output;

    /* get current pin data */
    output = inb( BASE );

    print_pins();

    /* Update pin data */
    if( state )
        output |= (int)pow( 2, pin -1 ); /* set */
    else
        output &= 255 - ((int)pow( 2, pin -1 )); /* reset */

    /* make it so */
    outb( output, BASE );
    
    print_pins();
}

void print_pins( void )
{
    uint output, x;

    output = inb( BASE ); /* read outputs */
    
    for( x = 0; x < 8; x++ )
    {
        printf( "%d", ( output & 1<<x ) != 0 );
    }

    printf( " => %d\n", output );
}
